==> grabinput-0.2.tar.gz: grabinput-0.2/src/grab.c <==
/*
 Grab all keyboard and possibly pointer input until ScrollLock pressed

 Copyright 2012  Andrey Tataranovich <tataranovich@gmail.com>

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "grab.h"

void showFullScreen( Display* dpy, Window wid )
{
	XWindowAttributes xwa;
	XGetWindowAttributes( dpy, wid, &xwa );

	XEvent e;
	e.xclient.type = ClientMessage;
	e.xclient.message_type = XInternAtom( dpy, "_NET_WM_STATE", False );
	e.xclient.display = dpy;
	e.xclient.window = wid;
	e.xclient.format = 32;
	e.xclient.data.l[0] = 1;
	e.xclient.data.l[1] = XInternAtom( dpy, "_NET_WM_STATE_FULLSCREEN", False );
	e.xclient.data.l[2] = 0;
	e.xclient.data.l[3] = 0;
	e.xclient.data.l[4] = 0;
	XSendEvent( dpy, xwa.root, False, SubstructureNotifyMask | SubstructureRedirectMask, &e );
}

int main(int argc, char* argv[]) {
	Display *display;
	XEvent ev;
	KeySym *keysym;
	Bool grabPointer = False;
	Bool showSaver = False;
	int i, opt;
	pid_t saverPID = -1;
	FILE* saverApp;

	while ((opt = getopt(argc, argv, "ls")) != -1) {
		switch (opt) {
			case 'l':
				grabPointer = True;
				break;
			case 's':
				saverApp = fopen(SCREENSAVER_APP, "r");
				if (saverApp != NULL) {
					showSaver = True;
					fclose(saverApp);
				} else {
					fprintf(stderr, "Unable to find screensaver application: %s - disabling screensaver\n", SCREENSAVER_APP);
					showSaver = False;
				}
				break;
			default:
				fprintf(stderr, "Usage: %s [-l]\n", argv[0]);
				exit(EXIT_FAILURE);
		}
	}

	display = XOpenDisplay(NULL);
	if (!display) {
		fprintf(stderr, "Unable to open display.\n");
		exit(EXIT_FAILURE);
	}

	for (i = 0; i<5; i++) {
		if (XGrabKeyboard(display, DefaultRootWindow(display), False, GrabModeAsync, GrabModeAsync, CurrentTime) != 0) {
			if (i == 4) {
				fprintf(stderr, "Failed to grab keyboard\n");
				XCloseDisplay(display);
				exit(EXIT_FAILURE);
			}
			usleep(500000);
		} else {
			break;
		}
	}

	if (grabPointer) {
		Cursor cursor = XCreateFontCursor(display, XC_pirate);
		for (i = 0; i < 5; i++) {
			if (XGrabPointer(display, DefaultRootWindow(display), False, ButtonPressMask, GrabModeAsync, GrabModeAsync, None, cursor, CurrentTime) != 0) {
				if (i == 4) {
					fprintf(stderr, "Failed to grab pointer\n");
					XUngrabKeyboard(display, CurrentTime);
					XCloseDisplay(display);
					exit(EXIT_FAILURE);
				}
				usleep(500000);
			} else {
				break;
			}
		}
	}

	if (showSaver) {
		int screen = DefaultScreen(display);
		int screenWidth = DisplayWidth(display, screen);
		int screenHeight = DisplayHeight(display, screen);

		Window saverWindow = XCreateSimpleWindow(display, RootWindow(display, screen), 0, 0, screenWidth, screenHeight, 0, 0, 0);
		XMapWindow(display, saverWindow);
		XSync(display, False);

		showFullScreen(display, saverWindow);

		char* saverArgv[128];
		char saverWinId[32];
		memset(&saverArgv, 0, sizeof(saverArgv));
		saverArgv[0] = SCREENSAVER_APP;
		saverArgv[1] = "-window-id";
		sprintf(saverWinId, "%d", (int) saverWindow);
		saverArgv[2] = saverWinId;
		saverArgv[3] = NULL;

		saverPID = fork();
		switch (saverPID) {
			case -1:
				fprintf(stderr, "Unable to fork(). Error: %s\n", strerror(errno));
				break;
			case 0:
				execvp(saverArgv[0], saverArgv);
				fprintf(stderr, "Unable to execute screensaver process. Error: %s\n", strerror(errno));
				break;
			default:
				break;
		}
	}

	while (1) {
		XNextEvent(display, &ev);
		switch(ev.type) {
			case KeyRelease:
				keysym = XGetKeyboardMapping(display, ev.xkey.keycode, 1, &i);
				if ((KeySym) *keysym == XK_Scroll_Lock) {
					XSync(display, False);
					XUngrabKeyboard(display, CurrentTime);
					if (grabPointer) {
						XUngrabPointer(display, CurrentTime);
					}
					XCloseDisplay(display);

					if (saverPID > 0) {
						fprintf(stderr, "Killing child process\n");
						kill(saverPID, SIGTERM);
					}
					exit(EXIT_SUCCESS);
				}
		}
	}
}

==> grabinput-0.3.tar.gz: grabinput-0.3/src/grab.c <==
/*
 Grab all keyboard and possibly pointer input until ScrollLock pressed

 Copyright 2012  Andrey Tataranovich <tataranovich@gmail.com>

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "grab.h"

void showFullScreen( Display* dpy, Window wid )
{
	XWindowAttributes xwa;
	XGetWindowAttributes( dpy, wid, &xwa );

	XEvent e;
	e.xclient.type = ClientMessage;
	e.xclient.message_type = XInternAtom( dpy, "_NET_WM_STATE", False );
	e.xclient.display = dpy;
	e.xclient.window = wid;
	e.xclient.format = 32;
	e.xclient.data.l[0] = 1;
	e.xclient.data.l[1] = XInternAtom( dpy, "_NET_WM_STATE_FULLSCREEN", False );
	e.xclient.data.l[2] = 0;
	e.xclient.data.l[3] = 0;
	e.xclient.data.l[4] = 0;
	XSendEvent( dpy, xwa.root, False, SubstructureNotifyMask | SubstructureRedirectMask, &e );
}

KeySym* getNextKeySym(Display* display) {
	XEvent xev;
	KeySym* keysym;
	int i;

	while (1) {
		XNextEvent(display, &xev);
		if (xev.type == KeyRelease) {
			keysym = XGetKeyboardMapping(display, xev.xkey.keycode, 1, &i);
			break;
		}
	}

	return keysym;
}

Bool checkMagickSequence(Display* display) {
	if ((KeySym) *getNextKeySym(display) == X_MAGICK_SEQ2) {
		if ((KeySym) *getNextKeySym(display) == X_MAGICK_SEQ3) {
			return True;
		}
	}
	return False;
}

int main(int argc, char* argv[]) {
	Display *display;
	Bool grabPointer = False;
	Bool showSaver = False;
	int i, opt;
	pid_t saverPID = -1;
	FILE* saverApp;

	while ((opt = getopt(argc, argv, "ls")) != -1) {
		switch (opt) {
			case 'l':
				grabPointer = True;
				break;
			case 's':
				saverApp = fopen(SCREENSAVER_APP, "r");
				if (saverApp != NULL) {
					showSaver = True;
					fclose(saverApp);
				} else {
					fprintf(stderr, "Unable to find screensaver application: %s - disabling screensaver\n", SCREENSAVER_APP);
					showSaver = False;
				}
				break;
			default:
				fprintf(stderr, "Usage: %s [-l]\n", argv[0]);
				exit(EXIT_FAILURE);
		}
	}

	display = XOpenDisplay(NULL);
	if (!display) {
		fprintf(stderr, "Unable to open display.\n");
		exit(EXIT_FAILURE);
	}

	for (i = 0; i<5; i++) {
		if (XGrabKeyboard(display, DefaultRootWindow(display), False, GrabModeAsync, GrabModeAsync, CurrentTime) != 0) {
			if (i == 4) {
				fprintf(stderr, "Failed to grab keyboard\n");
				XCloseDisplay(display);
				exit(EXIT_FAILURE);
			}
			usleep(500000);
		} else {
			break;
		}
	}

	if (grabPointer) {
		Cursor cursor = XCreateFontCursor(display, XC_pirate);
		for (i = 0; i < 5; i++) {
			if (XGrabPointer(display, DefaultRootWindow(display), False, ButtonPressMask, GrabModeAsync, GrabModeAsync, None, cursor, CurrentTime) != 0) {
				if (i == 4) {
					fprintf(stderr, "Failed to grab pointer\n");
					XUngrabKeyboard(display, CurrentTime);
					XCloseDisplay(display);
					exit(EXIT_FAILURE);
				}
				usleep(500000);
			} else {
				break;
			}
		}
	}

	if (showSaver) {
		int screen = DefaultScreen(display);
		int screenWidth = DisplayWidth(display, screen);
		int screenHeight = DisplayHeight(display, screen);

		Window saverWindow = XCreateSimpleWindow(display, RootWindow(display, screen), 0, 0, screenWidth, screenHeight, 0, 0, 0);
		XMapWindow(display, saverWindow);
		XSync(display, False);

		showFullScreen(display, saverWindow);

		char* saverArgv[128];
		char saverWinId[32];
		memset(&saverArgv, 0, sizeof(saverArgv));
		saverArgv[0] = SCREENSAVER_APP;
		saverArgv[1] = "-window-id";
		sprintf(saverWinId, "%d", (int) saverWindow);
		saverArgv[2] = saverWinId;
		saverArgv[3] = NULL;

		saverPID = fork();
		switch (saverPID) {
			case -1:
				fprintf(stderr, "Unable to fork(). Error: %s\n", strerror(errno));
				break;
			case 0:
				execvp(saverArgv[0], saverArgv);
				fprintf(stderr, "Unable to execute screensaver process. Error: %s\n", strerror(errno));
				exit(EXIT_FAILURE);
				break;
			default:
				break;
		}
	}

	while (1) {
		if ((KeySym) *getNextKeySym(display) == X_MAGICK_SEQ1 && checkMagickSequence(display)) {
			XSync(display, False);
			XUngrabKeyboard(display, CurrentTime);
			if (grabPointer) {
				XUngrabPointer(display, CurrentTime);
			}
			XCloseDisplay(display);

			if (saverPID > 0) {
				fprintf(stderr, "Killing child process\n");
				kill(saverPID, SIGTERM);
			}
			exit(EXIT_SUCCESS);
		}
	}
}
